// EloInterface.h
//
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the ELOMTAPI_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// ELOMT_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.

#pragma once
#define _NTDDK_
#include "mmioctl.h" // does include nested winioctl.h
#include "EloMtApi.h"

// -----------------------------------------------------------------------------
// Interface function calls 
// -----------------------------------------------------------------------------

#ifndef ELOPUBIF_EXPORTS

#ifdef __cplusplus
extern "C" {
#endif 

// This is the first call required to be made to enumerate & initialize 
// the touch screens. It returns the total number of touch screens found.
ELOMT_API int EloIF_EnumTouchScreen ();

// Get total number of touch screens for current system
ELOMT_API int EloIF_GetScreenCount() ;

// Enable or disable touch of specified screen by screen index
ELOMT_API bool EloIF_EnableTouch (int nScreenIndex, BOOL bEnableTouch);

// Get touch state of specified by screen index
ELOMT_API bool EloIF_IsTouchEnabled (int nScreenIndex);

// Get touch point from touch screen. *touch_status return touch status
// as defined in mmioctl.h (TOUCH_STATUS)
ELOMT_API int EloIF_GetTouchPacket (int nScreenIndex, int *x, int *y, int *z, int *touch_status);

// Get calibration data for a touch screen
ELOMT_API bool EloIF_GetCalData (int nScreenIndex, MMDevCalData * pCalData);

// Set calibration data for a touch screen
ELOMT_API bool EloIF_SetCalData (int nScreenIndex, MMDevCalData * pCalData);

// Get diagnostics data for a specified touch screen
ELOMT_API DWORD EloIF_GetDiagnosticsData(int nScreenIndex, MMDevIODIAGNOSTICS * pDiag);

// Get U Alignment counts (for IntelliTouch Plus, PID0xnn22 only)
ELOMT_API int EloIF_GetUAlignCounts (int nScreenIndex, int* u1_cycles, int* u2_cycles);

// Get the rectangle of current clipping area
ELOMT_API int EloIF_GetClipRectangles(int nScreenIndex, MMDevIO_CLIPPING_MODE* pClipMode);

// Set clipping rectangle bounds
ELOMT_API int EloIF_SetClipRectangles(int nScreenIndex, MMDevIO_CLIPPING_MODE* pClipMode);

// Get Full screen clipping rectangles
ELOMT_API int EloIF_GetFullScreenClipRectangle(int nScreenIndex, MMDevIO_FULL_SCREEN_CLIPPING* pFullScreenClipMode);

// Set Full screen clipping rectangles
ELOMT_API int EloIF_SetFullScreenClipRectangle(int nScreenIndex, MMDevIO_FULL_SCREEN_CLIPPING* pFullScreenClipMode);

// Get Edge Acceleration data for a touch screen
ELOMT_API int EloIF_GetAcceleration(int nScreenIndex, int w_now, int h_now, ACCELDATA *pEA);

// Set Edge Acceleration for a touch screen
ELOMT_API int EloIF_SetAcceleration(int nScreenIndex, int w_now, int h_now, const ACCELDATA * pAccel);

// Set debug level by a specified screen by screen index
ELOMT_API bool EloIF_SetDebugLevel (ULONG ulDbgLevels, DWORD screen_index);

// Get debug level by a specified screen by screen index
ELOMT_API DWORD EloIF_GetDebugLevel (DWORD screen_index);

// Enable touch sound
ELOMT_API bool EloIF_EnableBeep (bool bEnable);

// Get state of beep
ELOMT_API bool EloIF_IsBeapEnabled ();

// Get MONITOR strucu by windows monitor number
ELOMT_API MONITOR * EloIF_GetMonitorByWinMonNum (int nWinMonNum);

// Get MONITOR struct by a monitor index 
ELOMT_API MONITOR * EloIF_GetMonitorByIndex (int nMonitorIndex);

// Go to monitor given by the Windows handle and a monitor index
ELOMT_API void EloIF_GotoMonitor (HWND hWnd, int nMonitorIndex);

// Get SCREEN struct by a specified screen by screen index
ELOMT_API SCREEN* EloIF_GetScreenByIndex (int nScreenIndex);

// Get monitor index by windows monitor number
ELOMT_API int EloIF_GetMonitorIndexByWinMonNum (int nWinMonNum);

// Get Monitor count
ELOMT_API int EloIF_GetMonitorCount ();

// Get PulseTouch TrackerParams of a touch screen. Please make sure the size of the 
// array of integers should be at least MAX_TRACKER_PARAMS
ELOMT_API int EloIF_GetAprParameters (int nScreenIndex, int * pnParameters, int nParameterCount);

// Set Pulse Touch TrackerParams by given # of Parameter and a specified screen by screen index
ELOMT_API bool EloIF_SetAprParameters (int nScreenIndex, int * pnParameters, int nParameterCount);

// Get progress info for downloading PulseTouch calibration data
ELOMT_API int EloIF_GetAprFlashProgress (int nScreenIndex, int * cur_page);

// Get Serial number of PulseTouch 700X
ELOMT_API bool EloIF_GetApr700XSerialNumbers (int nScreenIndex, MMDevIOGET_SERIAL_NUMBERS *pSN);

// Flush the controller buffer by a specified screen by a screen index
ELOMT_API bool EloIF_FlushControllerSmartsetBuffer (int nScreenIndex);

// Get controller firmware version
ELOMT_API bool EloIF_GetControllerFWVersion (int nScreenIndex, TCHAR * ver, size_t nLen);

// Close all Elo device handles
ELOMT_API void EloIF_CloseEloDeviceHandles();

// Send Smartset command
ELOMT_API int EloIF_SendSmartsetCommand( PSMARTSETDATA pSmartsetData );

// Get Smartset response
ELOMT_API int EloIF_GetSmartsetResponse( PSMARTSETDATA pSmartsetData );

// Get Smartset response status 
ELOMT_API int EloIF_GetSmartsetResponseStatus( PSMARTSETDATA pSmartsetData );

// Apply transactional Smartset command to retrieve the response data
ELOMT_API BOOL EloIF_SmartsetTransaction(SMARTSETDATA* pSS, SMARTSET_PKT* pSSPACKET=NULL, int nNumPkts=0, int* pnRespPkts=NULL, int nDelay=20);

// Clear all pending GetPoint ioctls
ELOMT_API bool EloIF_ClearAllGetPoint ();

// Get IR internal sound setting by a specified screen of screen index
ELOMT_API bool EloIF_GetIRInternalSound( int nScreenIndex, UCHAR &State, DWORD &Freq, DWORD &Dura );

// Set IR internal sound setting by a specified screen of screeen index
ELOMT_API bool EloIF_SetIRInternalSound( int nScreenIndex, UCHAR State, DWORD Freq, DWORD Dura );

// Copy device parameters of a touch screen to Elo global registry keys
ELOMT_API bool EloIF_CopyDeviceParametersToGlobalKey (int nScreenIndex);

// Copy Elo global registry keys to device parameters of a touch screen
ELOMT_API bool EloIF_CopyGlobalKeyToDeviceParameters (int nScreenIndex);

// Apply Elo global registry keys to a touch screen
ELOMT_API bool EloIF_ApplyGlobalKeys( int nScreenIndex );

// Store device keys to Elo global registry keys
ELOMT_API bool EloIF_StoreDevKeysToGlobal( int nScreenIndex );

// Beep at the specified frequency and duration
ELOMT_API bool EloIF_MakeBeep(unsigned int frequency, unsigned int duration);

// Get multiple touche points from a touch screen
ELOMT_API bool EloIF_GetMultiTouch (int nScreenIndex, MT_TOUCH* pMtTouch);

// Locking mechanism to protect share object in Smartset command
// EloIF_AcquireSmartsetLock and EloIF_ReleaseSmartsetLock are used if you want to control
// Smartset transaction EloIF_SendSmartsetCommand and EloIF_GetSmartsetResponse. We strongly
// recommend to use EloIF_SmartsetTransaction since it is thread safe. Acquire Smartset 
// lock prevent from other Smartset command to enter controller
ELOMT_API int EloIF_AcquireSmartsetLock( int nScreenIndex, int nRetry = 1 );

// Release Smartset lock
ELOMT_API int EloIF_ReleaseSmartsetLock( int nScreenIndex );

#ifdef __cplusplus
}
#endif

#endif // ELOPUBIF_EXPORTS

